using System;

using SdlDotNet;
using Tao.OpenGl;

namespace Jumpers
{
	/// <summary>
	/// The main startup class.
	/// </summary>
	public class MainLoop
	{
		private static MainLoop m_instance;
		/// <summary>
		/// The global mainloop entry.
		/// </summary>
		public static MainLoop Instance
		{
			get { return m_instance; }
		}

		/// <summary>
		/// The "application running" flag.
		/// </summary>
		public bool Running;

		private IScreen m_game_screen;
		/// <summary>
		/// The current screen.
		/// </summary>
		public IScreen GameScreen
		{
			get { return m_game_screen; }
			set
			{
				m_game_screen = value;
				Running = ( m_screen != null );
			}
		}

		private Surface m_screen;

		public MainLoop()
		{
			m_instance = this;

			Running = true;
		}

		/// <summary>
		/// The main loop.
		/// </summary>
		public void Run()
		{
			Init();
			Reshape();

			int cur_ticks = Timer.Ticks;

			while ( Running )
			{
				while ( Events.Poll() ) { }

				// Update the frames (with frame compensation)
				int new_ticks = Timer.Ticks;
				int diff = new_ticks - cur_ticks;
                
				bool do_draw = false;

				if ( diff < 0 || diff > 1000 )
				{ // Skipped too much; reset counter
					cur_ticks = new_ticks;
				}
				else
				{
					int frames = (int) ( (float)diff / 16.6667f );
					if ( frames > 0 )
					{
						do_draw = true;
						cur_ticks = new_ticks;
					}

					while ( frames-- > 0 )
					{
						Idle();
					}
				}

				// Draw the current screen
				if ( do_draw )
					Draw();
			}
		}

		/// <summary>
		/// Inits the gfx.
		/// </summary>
		private void Init()
		{
			// Init the screen
#if DEBUG
			m_screen = Video.SetVideoModeWindowOpenGL( 640, 480, true );
#else
			m_screen = Video.SetVideoModeOpenGL( 1024, 768, 16 );
#endif
			Events.Quit += new QuitEventHandler( this.Quit );
			Events.VideoResize += new VideoResizeEventHandler( this.Resize );
			Events.Keyboard += new KeyboardEventHandler( this.KeyHandler );

			Video.WindowCaption = "Jumpers - LD48h #6 competition entry -";
			Video.Mouse.ShowCursor( false );

			// Init the OpenGL
			Gl.glClearColor( 0, 0, 0, 0 );
			Gl.glShadeModel( Gl.GL_SMOOTH );
			Gl.glEnable( Gl.GL_BLEND );

			// Init the fonter
			Fonter.Init();

			// Init the logic
			GameScreen = new MenuScreen();
		}

		/// <summary>
		/// New window size or exposure
		/// </summary>
		private void Reshape()
		{
			int new_w = m_screen.Width;
			int new_h = m_screen.Height;
			double h = (double)new_w / (double)new_h;

			Gl.glViewport( 0, 0, new_w, new_h );
			Gl.glMatrixMode( Gl.GL_PROJECTION );
			Gl.glLoadIdentity();
			Gl.glOrtho( 0, 640, 480, 0, -1, 1 );
			Gl.glMatrixMode( Gl.GL_MODELVIEW );
			Gl.glLoadIdentity();
		}

		/// <summary>
		/// Runs the game logic.
		/// </summary>
		private void Idle()
		{
			if ( GameScreen != null )
				GameScreen.Update( Video.Mouse );
		}

		/// <summary>
		/// Draws the game screen.
		/// </summary>
		private void Draw()
		{
			Gl.glClear( Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT );
			Gl.glLoadIdentity();

			if ( GameScreen != null )
				GameScreen.Draw();

			Video.GLSwapBuffers();
		}

		/// <summary>
		/// Quit event.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Quit( object sender, QuitEventArgs e )
		{
			Running = false;
		}

		/// <summary>
		/// Video resize event,
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Resize (object sender, VideoResizeEventArgs e)
		{
			m_screen = Video.SetVideoModeWindowOpenGL( e.W, e.H, true );
			Reshape();
		}

		/// <summary>
		/// Key event handler.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void KeyHandler( object sender, KeyboardEventArgs e )
		{
			if ( e.Key == Key.Escape )
			{
				Running = false;
				return;
			}

			if ( GameScreen == null )
				return;

			switch ( e.Type )
			{
				case EventTypes.KeyDown:
					GameScreen.KeyDown( e.Key );
					break;

				case EventTypes.KeyUp:
					GameScreen.KeyUp( e.Key );
					break;
			}
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			MainLoop main = new MainLoop();
			main.Run();
		}
	}
}
